"""
main
"""
import os
from typing import Dict, Any, List
from pytconf import register_endpoint, register_main, config_arg_parse_and_launch

from pypowerline.utils import execute_python_file
from pypowerline.segments import Segment
from pypowerline.colors import cprint, Color

from pypowerline.symbols import Symbol
from pypowerline.static import DESCRIPTION, APP_NAME, VERSION_STR


@register_endpoint(
    configs=[],
    description="print the location of pypowerlines config file",
)
def print_config_file() -> None:
    print(os.path.expanduser("~/.config/pypowerline/segments.py"))


@register_endpoint(
    configs=[],
    description="print a prompt for bash",
)
def bash() -> None:
    py_file = os.path.expanduser("~/.config/pypowerline/segments.py")
    vals: Dict[str, Any] = {}
    try:
        execute_python_file(py_file, vals=vals)
    # pylint: disable=broad-exception-caught
    except Exception as e:
        print(e)
        print("cannot execute segments > ", end="")
        return
    if "segments" not in vals:
        print("segments not defined > ", end="")
        return
    segments: List[Segment] = vals["segments"]
    for segment in segments:
        text = ""
        if segment.icon is not None:
            # TODO: why the two spaces here? Is it because it's unicode?
            text += segment.icon.value + "  "
        text += segment.get_text()
        if segment.color is None or segment.background is None:
            print(text, end="")
        else:
            cprint(
                text,
                segment.color,
                segment.background,
            )
        if segment.separator is not None:
            text = segment.separator.value
            if segment.color is None or segment.background is None:
                print(text, end="")
            else:
                attrs: List[str] = []
                if segment.reverse_sep:
                    attrs.append("reverse")
                    attrs.append("reset")
                cprint(
                    text,
                    segment.color,
                    segment.background,
                    _attrs=attrs,
                )


@register_endpoint(
    configs=[],
    description="do some test printing on the terminal (development purposes)",
)
def test() -> None:
    cwd = os.getcwd()
    symbol = Symbol.SEPARATOR_B.value
    print(f"{cwd} > ", end="")
    cprint("cwd", Color.BLACK, Color.LIGHT_MAGENTA)
    cprint("cwd", Color.BLACK, Color.LIGHT_GREY)
    cprint("cwd", Color.BLACK, Color.GREY)
    cprint("cwd", Color.BLACK, Color.BLUE)
    cprint("cwd", Color.BLACK, Color.LIGHT_BLUE)
    cprint("cwd", Color.WHITE, Color.CYAN)
    cprint("cwd", Color.WHITE, Color.GREEN)
    cprint(f"{symbol}", Color.LIGHT_CYAN, Color.BLACK)
    print()


@register_endpoint(
    configs=[],
    description="do tmux",
)
def tmux() -> None:
    cwd = os.getcwd()
    print(f"{cwd} > ", end="")
    cprint(f"{cwd}", Color.RED, Color.LIGHT_MAGENTA)


@register_endpoint(
    configs=[],
    description="print special symbols (used for development)",
)
def dump_symbols() -> None:
    for symbol in Symbol:
        print(symbol, symbol.value)


@register_main(
    main_description=DESCRIPTION,
    app_name=APP_NAME,
    version=VERSION_STR,
)
def main():
    config_arg_parse_and_launch()


if __name__ == "__main__":
    main()



# https://www.ditig.com/256-colors-cheat-sheet
# https://en.wikipedia.org/wiki/X11_color_names
# https://vim.fandom.com/wiki/Xterm256_color_names_for_console_Vim
# https://ss64.com/bash/syntax-colors.html
# https://en.wikipedia.org/wiki/X11_color_names
# https://en.wikipedia.org/wiki/Web_colors
# https://unix.stackexchange.com/questions/148/colorizing-your-terminal-and-shell-environment

xterm_256_colors = []

# Generate the color cube (216 colors)
for r in range(0, 6):
    for g in range(0, 6):
        for b in range(0, 6):
            xterm_256_colors.append((r * 40, g * 40, b * 40))

# Generate the grayscale ramp (24 colors)
for shade in range(0, 24):
    gray_value = shade * 10 + 8
    xterm_256_colors.append((gray_value, gray_value, gray_value))

# Now, xterm_256_colors contains a list of all Xterm-256 colors

# Example: Print the first 16 colors in the list
for i, color in enumerate(xterm_256_colors):
    print(f"Color {i}: RGB{color}")



from abc import ABC, abstractmethod
from typing import Optional

import os
import os.path

from pypowerline.symbols import Symbol
from pypowerline.colors import Color


class Segment(ABC):
    def __init__(
            self,
            color: Optional[Color] = None,
            background: Optional[Color] = None,
            icon: Optional[Symbol] = None,
            separator: Optional[Symbol] = Symbol.SEPARATOR,
            reverse_sep: bool = True,
    ):
        self.color = color
        self.background = background
        self.icon = icon
        self.separator = separator
        self.reverse_sep = reverse_sep

    @abstractmethod
    def get_text(self) -> str:
        """ return the text of the segment """


class SegmentCwd(Segment):
    def __init__(
            self,
            color: Optional[Color] = None,
            background: Optional[Color] = None,
            icon: Optional[Symbol] = Symbol.FOLDER,
            separator: Optional[Symbol] = Symbol.SEPARATOR_B,
            reverse_sep: bool = True,
            last_only: bool = False,
            home_as_tilde: bool = True,
    ):
        super().__init__(
            color=color,
            background=background,
            icon=icon,
            separator=separator,
            reverse_sep=reverse_sep,
        )
        self.last_only = last_only
        self.home_as_tilde = home_as_tilde
        self.home_directory = os.path.expanduser("~")

    def get_text(self) -> str:
        cwd = os.getcwd()
        if self.home_as_tilde and cwd.startswith(self.home_directory):
            cwd = "~" + cwd[len(self.home_directory):]
        if self.last_only:
            return os.path.basename(cwd)
        return cwd


class SegmentForward(Segment):
    def __init__(self):
        super().__init__()

    def get_text(self):
        return " > "


class SegmentSpace(Segment):
    def __init__(self):
        super().__init__(separator=None)

    def get_text(self):
        return " "



from enum import Enum


class Color(Enum):
    BLACK = "black"
    GREY = "grey"
    RED = "red"
    GREEN = "green"
    YELLOW = "yellow"
    BLUE = "blue"
    MAGENTA = "magenta"
    CYAN = "cyan"
    LIGHT_GREY = "light_grey"
    DARK_GREY = "dark_grey"
    LIGHT_RED = "light_red"
    LIGHT_GREEN = "light_green"
    LIGHT_YELLOW = "light_yellow"
    LIGHT_BLUE = "light_blue"
    LIGHT_MAGENTA = "light_magenta"
    LIGHT_CYAN = "light_cyan"
    WHITE = "white"


def cprint(_text: str, _foreground: Color, _background: Color, _attrs=None):
    pass
